import { addUniqueNumber } from 'fast-unique-numbers';
import { isSupported } from 'standardized-audio-context';
import { render } from './helpers/render';
/*
 * @todo Explicitly referencing the barrel file seems to be necessary when enabling the
 * isolatedModules compiler option.
 */
export * from './types/index';
export { isSupported };
export const load = (url) => {
    const worker = new Worker(url);
    const ongoingRecordingRequests = new Set();
    const analyze = (...args) => {
        var _a;
        const [audioBuffer, offsetOrTempoSettings, durationOrTempoSettings] = args;
        const offset = typeof offsetOrTempoSettings === 'number' ? offsetOrTempoSettings : 0;
        const duration = typeof durationOrTempoSettings === 'number' ? durationOrTempoSettings : audioBuffer.duration - offset;
        const tempoSettings = typeof offsetOrTempoSettings === 'object'
            ? offsetOrTempoSettings
            : typeof durationOrTempoSettings === 'object'
                ? durationOrTempoSettings
                : (_a = args[3]) !== null && _a !== void 0 ? _a : null;
        return new Promise(async (resolve, reject) => {
            const { channelData, sampleRate } = await render(audioBuffer, offset, duration);
            const id = addUniqueNumber(ongoingRecordingRequests);
            const onMessage = ({ data }) => {
                if (data.id === id) {
                    ongoingRecordingRequests.delete(id);
                    worker.removeEventListener('message', onMessage);
                    if (data.error === null) {
                        resolve(data.result.tempo);
                    }
                    else {
                        reject(new Error(data.error.message));
                    }
                }
            };
            worker.addEventListener('message', onMessage);
            worker.postMessage({
                id,
                method: 'analyze',
                params: { channelData, sampleRate, ...(tempoSettings === null ? tempoSettings : { tempoSettings }) }
            }, [channelData.buffer]);
        });
    };
    const guess = (...args) => {
        var _a;
        const [audioBuffer, offsetOrTempoSettings, durationOrTempoSettings] = args;
        const offset = typeof offsetOrTempoSettings === 'number' ? offsetOrTempoSettings : 0;
        const duration = typeof durationOrTempoSettings === 'number' ? durationOrTempoSettings : audioBuffer.duration - offset;
        const tempoSettings = typeof offsetOrTempoSettings === 'object'
            ? offsetOrTempoSettings
            : typeof durationOrTempoSettings === 'object'
                ? durationOrTempoSettings
                : (_a = args[3]) !== null && _a !== void 0 ? _a : null;
        return new Promise(async (resolve, reject) => {
            const { channelData, sampleRate } = await render(audioBuffer, offset, duration);
            const id = addUniqueNumber(ongoingRecordingRequests);
            const onMessage = ({ data }) => {
                if (data.id === id) {
                    ongoingRecordingRequests.delete(id);
                    worker.removeEventListener('message', onMessage);
                    if (data.error === null) {
                        resolve(data.result);
                    }
                    else {
                        reject(new Error(data.error.message));
                    }
                }
            };
            worker.addEventListener('message', onMessage);
            worker.postMessage({
                id,
                method: 'guess',
                params: { channelData, sampleRate, ...(tempoSettings === null ? tempoSettings : { tempoSettings }) }
            }, [channelData.buffer]);
        });
    };
    return {
        analyze,
        guess
    };
};
//# sourceMappingURL=module.js.map